@@ -252,14 +252,6 @@ BOOL WINAPI HeapUnlock(
 }
 
 
-/***********************************************************************
- *           GetProcessHeaps    (KERNEL32.@)
- */
-DWORD WINAPI GetProcessHeaps( DWORD count, HANDLE *heaps )
-{
-    return RtlGetProcessHeaps( count, heaps );
-}
-
 
 /* These are needed so that we can call the functions from inside kernel itself */
 
@@ -1491,3 +1483,11 @@ BOOL WINAPI FreeUserPhysicalPages(HANDLE process, ULONG_PTR *pages, ULONG_PTR *u
     SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
     return FALSE;
 }
+
+BOOL WINAPI MapUserPhysicalPages(PVOID addr, ULONG_PTR page_count, PULONG_PTR pages)
+{
+    FIXME("(%p, %lu, %p): stub\n", addr, page_count, pages);
+    *pages = 0;
+    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+    return FALSE;
+}
