@@ -41,6 +41,7 @@
 
 #include "wine/exception.h"
 #include "wine/list.h"
+#include "wine/asm.h"
 #include "wine/debug.h"
 #include "wine/unicode.h"
 
@@ -232,33 +233,6 @@ BOOL WINAPI SetDefaultDllDirectories( DWORD flags )
 }
 
 
-/****************************************************************************
- *              DisableThreadLibraryCalls (KERNEL32.@)
- *
- * Inform the module loader that thread notifications are not required for a dll.
- *
- * PARAMS
- *  hModule [I] Module handle to skip calls for
- *
- * RETURNS
- *  Success: TRUE. Thread attach and detach notifications will not be sent
- *           to hModule.
- *  Failure: FALSE. Use GetLastError() to determine the cause.
- *
- * NOTES
- *  This is typically called from the dll entry point of a dll during process
- *  attachment, for dlls that do not need to process thread notifications.
- */
-BOOL WINAPI DisableThreadLibraryCalls( HMODULE hModule )
-{
-    NTSTATUS    nts = LdrDisableThreadCalloutsForDll( hModule );
-    if (nts == STATUS_SUCCESS) return TRUE;
-
-    SetLastError( RtlNtStatusToDosError( nts ) );
-    return FALSE;
-}
-
-
 /***********************************************************************
  *             GetBinaryTypeW                     [KERNEL32.@]
  *
@@ -401,194 +375,6 @@ BOOL WINAPI GetBinaryTypeA( LPCSTR lpApplicationName, LPDWORD lpBinaryType )
     return FALSE;
 }
 
-/***********************************************************************
- *              GetModuleHandleExA         (KERNEL32.@)
- */
-BOOL WINAPI GetModuleHandleExA( DWORD flags, LPCSTR name, HMODULE *module )
-{
-    WCHAR *nameW;
-
-    if (!name || (flags & GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS))
-        return GetModuleHandleExW( flags, (LPCWSTR)name, module );
-
-    if (!(nameW = FILE_name_AtoW( name, FALSE ))) return FALSE;
-    return GetModuleHandleExW( flags, nameW, module );
-}
-
-/***********************************************************************
- *              GetModuleHandleExW         (KERNEL32.@)
- */
-BOOL WINAPI GetModuleHandleExW( DWORD flags, LPCWSTR name, HMODULE *module )
-{
-    NTSTATUS status = STATUS_SUCCESS;
-    HMODULE ret;
-    ULONG_PTR magic;
-    BOOL lock;
-
-    if (!module)
-    {
-        SetLastError( ERROR_INVALID_PARAMETER );
-        return FALSE;
-    }
-
-    /* if we are messing with the refcount, grab the loader lock */
-    lock = (flags & GET_MODULE_HANDLE_EX_FLAG_PIN) || !(flags & GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT);
-    if (lock)
-        LdrLockLoaderLock( 0, NULL, &magic );
-
-    if (!name)
-    {
-        ret = NtCurrentTeb()->Peb->ImageBaseAddress;
-    }
-    else if (flags & GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS)
-    {
-        void *dummy;
-        if (!(ret = RtlPcToFileHeader( (void *)name, &dummy ))) status = STATUS_DLL_NOT_FOUND;
-    }
-    else
-    {
-        UNICODE_STRING wstr;
-        RtlInitUnicodeString( &wstr, name );
-        status = LdrGetDllHandle( NULL, 0, &wstr, &ret );
-    }
-
-    if (status == STATUS_SUCCESS)
-    {
-        if (flags & GET_MODULE_HANDLE_EX_FLAG_PIN)
-            LdrAddRefDll( LDR_ADDREF_DLL_PIN, ret );
-        else if (!(flags & GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT))
-            LdrAddRefDll( 0, ret );
-    }
-    else SetLastError( RtlNtStatusToDosError( status ) );
-
-    if (lock)
-        LdrUnlockLoaderLock( 0, magic );
-
-    if (status == STATUS_SUCCESS) *module = ret;
-    else *module = NULL;
-
-    return (status == STATUS_SUCCESS);
-}
-
-/***********************************************************************
- *              GetModuleHandleA         (KERNEL32.@)
- *
- * Get the handle of a dll loaded into the process address space.
- *
- * PARAMS
- *  module [I] Name of the dll
- *
- * RETURNS
- *  Success: A handle to the loaded dll.
- *  Failure: A NULL handle. Use GetLastError() to determine the cause.
- */
-HMODULE WINAPI DECLSPEC_HOTPATCH GetModuleHandleA(LPCSTR module)
-{
-    HMODULE ret;
-
-    GetModuleHandleExA( GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, module, &ret );
-    return ret;
-}
-
-/***********************************************************************
- *		GetModuleHandleW (KERNEL32.@)
- *
- * Unicode version of GetModuleHandleA.
- */
-HMODULE WINAPI GetModuleHandleW(LPCWSTR module)
-{
-    HMODULE ret;
-
-    GetModuleHandleExW( GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, module, &ret );
-    return ret;
-}
-
-
-/***********************************************************************
- *              GetModuleFileNameA      (KERNEL32.@)
- *
- * Get the file name of a loaded module from its handle.
- *
- * RETURNS
- *  Success: The length of the file name, excluding the terminating NUL.
- *  Failure: 0. Use GetLastError() to determine the cause.
- *
- * NOTES
- *  This function always returns the long path of hModule
- *  The function doesn't write a terminating '\0' if the buffer is too 
- *  small.
- */
-DWORD WINAPI GetModuleFileNameA(
-	HMODULE hModule,	/* [in] Module handle (32 bit) */
-	LPSTR lpFileName,	/* [out] Destination for file name */
-        DWORD size )		/* [in] Size of lpFileName in characters */
-{
-    LPWSTR filenameW = HeapAlloc( GetProcessHeap(), 0, size * sizeof(WCHAR) );
-    DWORD len;
-
-    if (!filenameW)
-    {
-        SetLastError( ERROR_NOT_ENOUGH_MEMORY );
-        return 0;
-    }
-    if ((len = GetModuleFileNameW( hModule, filenameW, size )))
-    {
-    	len = FILE_name_WtoA( filenameW, len, lpFileName, size );
-        if (len < size)
-            lpFileName[len] = '\0';
-        else
-            SetLastError( ERROR_INSUFFICIENT_BUFFER );
-    }
-    HeapFree( GetProcessHeap(), 0, filenameW );
-    return len;
-}
-
-/***********************************************************************
- *              GetModuleFileNameW      (KERNEL32.@)
- *
- * Unicode version of GetModuleFileNameA.
- */
-DWORD WINAPI GetModuleFileNameW( HMODULE hModule, LPWSTR lpFileName, DWORD size )
-{
-    ULONG len = 0;
-    ULONG_PTR magic;
-    LDR_MODULE *pldr;
-    NTSTATUS nts;
-    WIN16_SUBSYSTEM_TIB *win16_tib;
-
-    if (!hModule && ((win16_tib = NtCurrentTeb()->Tib.SubSystemTib)) && win16_tib->exe_name)
-    {
-        len = min(size, win16_tib->exe_name->Length / sizeof(WCHAR));
-        memcpy( lpFileName, win16_tib->exe_name->Buffer, len * sizeof(WCHAR) );
-        if (len < size) lpFileName[len] = '\0';
-        goto done;
-    }
-
-    LdrLockLoaderLock( 0, NULL, &magic );
-
-    if (!hModule) hModule = NtCurrentTeb()->Peb->ImageBaseAddress;
-    nts = LdrFindEntryForAddress( hModule, &pldr );
-    if (nts == STATUS_SUCCESS)
-    {
-        len = min(size, pldr->FullDllName.Length / sizeof(WCHAR));
-        memcpy(lpFileName, pldr->FullDllName.Buffer, len * sizeof(WCHAR));
-        if (len < size)
-        {
-            lpFileName[len] = '\0';
-            SetLastError( 0 );
-        }
-        else
-            SetLastError( ERROR_INSUFFICIENT_BUFFER );
-    }
-    else SetLastError( RtlNtStatusToDosError( nts ) );
-
-    LdrUnlockLoaderLock( 0, magic );
-done:
-    TRACE( "%s\n", debugstr_wn(lpFileName, len) );
-    return len;
-}
-
-
 /***********************************************************************
  *           get_dll_system_path
  */
@@ -940,11 +726,20 @@ static HMODULE load_library( const UNICODE_STRING *libname, DWORD flags )
     const DWORD unsupported_flags = (LOAD_IGNORE_CODE_AUTHZ_LEVEL |
                                      LOAD_LIBRARY_REQUIRE_SIGNED_TARGET);
 
-    if (!(flags & load_library_search_flags)) flags |= default_search_flags;
-
     if( flags & unsupported_flags)
         FIXME("unsupported flag(s) used (flags: 0x%08x)\n", flags);
 
+    if (flags & LOAD_WITH_ALTERED_SEARCH_PATH)
+    {
+        if (flags & load_library_search_flags)
+        {
+            SetLastError( ERROR_INVALID_PARAMETER );
+            return 0;
+        }
+        if (default_search_flags) flags |= default_search_flags | LOAD_LIBRARY_SEARCH_DLL_LOAD_DIR;
+    }
+    else if (!(flags & load_library_search_flags)) flags |= default_search_flags;
+
     if (flags & load_library_search_flags)
         load_path = get_dll_load_path_search_flags( libname->Buffer, flags );
     else
@@ -1185,11 +980,15 @@ extern FARPROC get_proc_address_wrapper( HMODULE module, LPCSTR function );
 
 __ASM_GLOBAL_FUNC( get_proc_address_wrapper,
                    "pushq %rbp\n\t"
+                   __ASM_SEH(".seh_pushreg %rbp\n\t")
                    __ASM_CFI(".cfi_adjust_cfa_offset 8\n\t")
                    __ASM_CFI(".cfi_rel_offset %rbp,0\n\t")
                    "movq %rsp,%rbp\n\t"
+                   __ASM_SEH(".seh_setframe %rbp,0\n\t")
                    __ASM_CFI(".cfi_def_cfa_register %rbp\n\t")
                    "subq $0x40,%rsp\n\t"
+                   __ASM_SEH(".seh_stackalloc 0x40\n\t")
+                   __ASM_SEH(".seh_endprologue\n\t")
                    "movaps %xmm0,-0x10(%rbp)\n\t"
                    "movaps %xmm1,-0x20(%rbp)\n\t"
                    "movaps %xmm2,-0x30(%rbp)\n\t"
@@ -1199,7 +998,7 @@ __ASM_GLOBAL_FUNC( get_proc_address_wrapper,
                    "movaps -0x30(%rbp), %xmm2\n\t"
                    "movaps -0x20(%rbp), %xmm1\n\t"
                    "movaps -0x10(%rbp), %xmm0\n\t"
-                   "movq %rbp,%rsp\n\t"
+                   "leaq 0(%rbp),%rsp\n\t"
                    __ASM_CFI(".cfi_def_cfa_register %rsp\n\t")
                    "popq %rbp\n\t"
                    __ASM_CFI(".cfi_adjust_cfa_offset -8\n\t")
