@@ -15,7 +15,6 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
-#include "config.h"
 
 #include <stdarg.h>
 
@@ -69,6 +68,7 @@ struct media_session
     LONG refcount;
     IMFMediaEventQueue *event_queue;
     IMFPresentationClock *clock;
+    IMFRateControl *clock_rate_control;
     struct list topologies;
     enum session_state state;
     CRITICAL_SECTION cs;
@@ -316,6 +316,8 @@ static ULONG WINAPI mfsession_Release(IMFMediaSession *iface)
             IMFMediaEventQueue_Release(session->event_queue);
         if (session->clock)
             IMFPresentationClock_Release(session->clock);
+        if (session->clock_rate_control)
+            IMFRateControl_Release(session->clock_rate_control);
         DeleteCriticalSection(&session->cs);
         heap_free(session);
     }
@@ -679,7 +681,7 @@ static HRESULT WINAPI session_rate_support_GetFastestRate(IMFRateSupport *iface,
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI session_rate_support_IsSupported(IMFRateSupport *iface, BOOL thin, float rate,
+static HRESULT WINAPI session_rate_support_IsRateSupported(IMFRateSupport *iface, BOOL thin, float rate,
         float *nearest_supported_rate)
 {
     FIXME("%p, %d, %f, %p.\n", iface, thin, rate, nearest_supported_rate);
@@ -694,7 +696,7 @@ static const IMFRateSupportVtbl session_rate_support_vtbl =
     session_rate_support_Release,
     session_rate_support_GetSlowestRate,
     session_rate_support_GetFastestRate,
-    session_rate_support_IsSupported,
+    session_rate_support_IsRateSupported,
 };
 
 static HRESULT WINAPI session_rate_control_QueryInterface(IMFRateControl *iface, REFIID riid, void **obj)
@@ -724,9 +726,11 @@ static HRESULT WINAPI session_rate_control_SetRate(IMFRateControl *iface, BOOL t
 
 static HRESULT WINAPI session_rate_control_GetRate(IMFRateControl *iface, BOOL *thin, float *rate)
 {
-    FIXME("%p, %p, %p.\n", iface, thin, rate);
+    struct media_session *session = impl_session_from_IMFRateControl(iface);
 
-    return E_NOTIMPL;
+    TRACE("%p, %p, %p.\n", iface, thin, rate);
+
+    return IMFRateControl_GetRate(session->clock_rate_control, thin, rate);
 }
 
 static const IMFRateControlVtbl session_rate_control_vtbl =
@@ -770,6 +774,12 @@ HRESULT WINAPI MFCreateMediaSession(IMFAttributes *config, IMFMediaSession **ses
     if (FAILED(hr = MFCreatePresentationClock(&object->clock)))
         goto failed;
 
+    if (FAILED(hr = IMFPresentationClock_QueryInterface(object->clock, &IID_IMFRateControl,
+            (void **)&object->clock_rate_control)))
+    {
+        goto failed;
+    }
+
     *session = &object->IMFMediaSession_iface;
 
     return S_OK;
