@@ -75,17 +75,23 @@ static inline int pf_output_stringW( pf_output *out, LPCWSTR str, int len )
     if( out->unicode )
     {
         LPWSTR p = out->buf.W + out->used;
-        out->used += len;
 
         if (!out->buf.W)
+        {
+            out->used += len;
             return len;
+        }
         if( space >= len )
         {
             memcpy( p, str, len*sizeof(WCHAR) );
+            out->used += len;
             return len;
         }
         if( space > 0 )
+        {
             memcpy( p, str, space*sizeof(WCHAR) );
+            out->used += space;
+        }
     }
     else
     {
@@ -93,16 +99,23 @@ static inline int pf_output_stringW( pf_output *out, LPCWSTR str, int len )
         ULONG n;
 
         RtlUnicodeToMultiByteSize( &n, str, len * sizeof(WCHAR) );
-        out->used += n;
 
         if (!out->buf.A)
+        {
+            out->used += n;
             return len;
+        }
         if( space >= n )
         {
             RtlUnicodeToMultiByteN( p, n, NULL, str, len * sizeof(WCHAR) );
+            out->used += n;
             return len;
         }
-        if (space > 0) RtlUnicodeToMultiByteN( p, space, NULL, str, len * sizeof(WCHAR) );
+        if (space > 0)
+        {
+            RtlUnicodeToMultiByteN( p, space, NULL, str, len * sizeof(WCHAR) );
+            out->used += space;
+        }
     }
     return -1;
 }
@@ -116,17 +129,23 @@ static inline int pf_output_stringA( pf_output *out, LPCSTR str, int len )
     if( !out->unicode )
     {
         LPSTR p = out->buf.A + out->used;
-        out->used += len;
 
         if (!out->buf.A)
+        {
+            out->used += len;
             return len;
+        }
         if( space >= len )
         {
             memcpy( p, str, len );
+            out->used += len;
             return len;
         }
         if( space > 0 )
+        {
             memcpy( p, str, space );
+            out->used += space;
+        }
     }
     else
     {
@@ -134,16 +153,23 @@ static inline int pf_output_stringA( pf_output *out, LPCSTR str, int len )
         ULONG n;
 
         RtlMultiByteToUnicodeSize( &n, str, len );
-        out->used += n / sizeof(WCHAR);
 
         if (!out->buf.W)
+        {
+            out->used += n / sizeof(WCHAR);
             return len;
+        }
         if (space >= n / sizeof(WCHAR))
         {
             RtlMultiByteToUnicodeN( p, n, NULL, str, len );
+            out->used += n / sizeof(WCHAR);
             return len;
         }
-        if (space > 0) RtlMultiByteToUnicodeN( p, space * sizeof(WCHAR), NULL, str, len );
+        if (space > 0)
+        {
+            RtlMultiByteToUnicodeN( p, space * sizeof(WCHAR), NULL, str, len );
+            out->used += space;
+        }
     }
     return -1;
 }
@@ -378,7 +404,7 @@ static void pf_fixup_exponent( char *buf )
 {
     char* tmp = buf;
 
-    while (tmp[0] && toupper(tmp[0]) != 'E')
+    while (tmp[0] && NTDLL_tolower(tmp[0]) != 'e')
         tmp++;
 
     if (tmp[0] && (tmp[1] == '+' || tmp[1] == '-') &&
@@ -637,7 +663,7 @@ static int pf_vsnprintf( pf_output *out, const WCHAR *format, __ms_va_list valis
             if( pf_is_double_format( flags.Format ) )
             {
                 sprintf( x, fmt, va_arg(valist, double) );
-                if (toupper(flags.Format) == 'E' || toupper(flags.Format) == 'G')
+                if (NTDLL_tolower(flags.Format) == 'e' || NTDLL_tolower(flags.Format) == 'g')
                     pf_fixup_exponent( x );
             }
             else
@@ -657,9 +683,7 @@ static int pf_vsnprintf( pf_output *out, const WCHAR *format, __ms_va_list valis
 
     /* check we reached the end, and null terminate the string */
     assert( *p == 0 );
-    pf_output_stringW( out, p, 1 );
-
-    return out->used - 1;
+    return out->used;
 }
 
 
@@ -686,6 +710,7 @@ int CDECL NTDLL__vsnprintf( char *str, SIZE_T len, const char *format, __ms_va_l
     }
     r = pf_vsnprintf( &out, formatW, args );
     RtlFreeHeap( GetProcessHeap(), 0, formatW );
+    if (out.used < len) str[out.used] = 0;
     return r;
 }
 
@@ -696,13 +721,16 @@ int CDECL NTDLL__vsnprintf( char *str, SIZE_T len, const char *format, __ms_va_l
 int CDECL NTDLL__vsnwprintf( WCHAR *str, SIZE_T len, const WCHAR *format, __ms_va_list args )
 {
     pf_output out;
+    int r;
 
     out.unicode = TRUE;
     out.buf.W = str;
     out.used = 0;
     out.len = len;
 
-    return pf_vsnprintf( &out, format, args );
+    r = pf_vsnprintf( &out, format, args );
+    if (out.used < len) str[out.used] = 0;
+    return r;
 }
 
 
@@ -736,6 +764,51 @@ int WINAPIV NTDLL__snwprintf( WCHAR *str, SIZE_T len, const WCHAR *format, ... )
 }
 
 
+/*********************************************************************
+ *                  _vsnprintf_s   (NTDLL.@)
+ */
+int CDECL _vsnprintf_s( char *str, SIZE_T size, SIZE_T len, const char *format, __ms_va_list args )
+{
+    DWORD sz;
+    LPWSTR formatW = NULL;
+    pf_output out;
+    int r;
+
+    out.unicode = FALSE;
+    out.buf.A = str;
+    out.used = 0;
+    out.len = min( size, len );
+
+    if (format)
+    {
+        RtlMultiByteToUnicodeSize( &sz, format, strlen(format) + 1 );
+        if (!(formatW = RtlAllocateHeap( GetProcessHeap(), 0, sz ))) return -1;
+        RtlMultiByteToUnicodeN( formatW, sz, NULL, format, strlen(format) + 1 );
+    }
+    r = pf_vsnprintf( &out, formatW, args );
+    RtlFreeHeap( GetProcessHeap(), 0, formatW );
+    if (out.used < size) str[out.used] = 0;
+    else str[0] = 0;
+    if (r == size) r = -1;
+    return r;
+}
+
+
+/*********************************************************************
+ *                  _snprintf_s   (NTDLL.@)
+ */
+int WINAPIV _snprintf_s( char *str, SIZE_T size, SIZE_T len, const char *format, ... )
+{
+    int ret;
+    __ms_va_list valist;
+
+    __ms_va_start( valist, format );
+    ret = _vsnprintf_s( str, size, len, format, valist );
+    __ms_va_end( valist );
+    return ret;
+}
+
+
 /*********************************************************************
  *                  vsprintf   (NTDLL.@)
  */
