@@ -35,27 +35,20 @@
 WINE_DEFAULT_DEBUG_CHANNEL(qcap);
 
 typedef struct {
-    IUnknown IUnknown_iface;
-    IUnknown *outerUnknown;
-    BaseFilter filter;
+    struct strmbase_filter filter;
     IPersistPropertyBag IPersistPropertyBag_iface;
     BaseOutputPin *output;
 } AudioRecord;
 
-static inline AudioRecord *impl_from_IUnknown(IUnknown *iface)
-{
-    return CONTAINING_RECORD(iface, AudioRecord, IUnknown_iface);
-}
-
-static inline AudioRecord *impl_from_BaseFilter(BaseFilter *filter)
+static inline AudioRecord *impl_from_strmbase_filter(struct strmbase_filter *filter)
 {
     return CONTAINING_RECORD(filter, AudioRecord, filter);
 }
 
 static inline AudioRecord *impl_from_IBaseFilter(IBaseFilter *iface)
 {
-    BaseFilter *filter = CONTAINING_RECORD(iface, BaseFilter, IBaseFilter_iface);
-    return impl_from_BaseFilter(filter);
+    struct strmbase_filter *filter = CONTAINING_RECORD(iface, struct strmbase_filter, IBaseFilter_iface);
+    return impl_from_strmbase_filter(filter);
 }
 
 static inline AudioRecord *impl_from_IPersistPropertyBag(IPersistPropertyBag *iface)
@@ -63,74 +56,6 @@ static inline AudioRecord *impl_from_IPersistPropertyBag(IPersistPropertyBag *if
     return CONTAINING_RECORD(iface, AudioRecord, IPersistPropertyBag_iface);
 }
 
-static HRESULT WINAPI Unknown_QueryInterface(IUnknown *iface, REFIID riid, LPVOID *ppv)
-{
-    AudioRecord *This = impl_from_IUnknown(iface);
-    if (IsEqualIID(riid, &IID_IUnknown)) {
-        TRACE("(%p)->(IID_IUnknown, %p)\n", This, ppv);
-        *ppv = &This->filter.IBaseFilter_iface;
-    } else if (IsEqualIID(riid, &IID_IPersist)) {
-        TRACE("(%p)->(IID_IPersist, %p)\n", This, ppv);
-        *ppv = &This->filter.IBaseFilter_iface;
-    } else if (IsEqualIID(riid, &IID_IMediaFilter)) {
-        TRACE("(%p)->(IID_IMediaFilter, %p)\n", This, ppv);
-        *ppv = &This->filter.IBaseFilter_iface;
-    } else if (IsEqualIID(riid, &IID_IBaseFilter)) {
-        TRACE("(%p)->(IID_IBaseFilter, %p)\n", This, ppv);
-        *ppv = &This->filter.IBaseFilter_iface;
-    } else if (IsEqualIID(riid, &IID_IPersistPropertyBag)) {
-        TRACE("(%p)->(IID_IPersistPropertyBag, %p)\n", This, ppv);
-        *ppv = &This->IPersistPropertyBag_iface;
-    } else {
-        FIXME("(%p): no interface for %s\n", This, debugstr_guid(riid));
-        *ppv = NULL;
-        return E_NOINTERFACE;
-    }
-    IUnknown_AddRef((IUnknown*)*ppv);
-    return S_OK;
-}
-
-static ULONG WINAPI Unknown_AddRef(IUnknown *iface)
-{
-    AudioRecord *This = impl_from_IUnknown(iface);
-    return BaseFilterImpl_AddRef(&This->filter.IBaseFilter_iface);
-}
-
-static ULONG WINAPI Unknown_Release(IUnknown *iface)
-{
-    AudioRecord *This = impl_from_IUnknown(iface);
-    ULONG ref = BaseFilterImpl_Release(&This->filter.IBaseFilter_iface);
-    TRACE("(%p/%p)->() ref=%d\n", iface, This, ref);
-    if (!ref) {
-        CoTaskMemFree(This);
-    }
-    return ref;
-}
-
-static const IUnknownVtbl UnknownVtbl = {
-    Unknown_QueryInterface,
-    Unknown_AddRef,
-    Unknown_Release
-};
-
-static HRESULT WINAPI AudioRecord_QueryInterface(IBaseFilter *iface, REFIID riid, void **ppv)
-{
-    AudioRecord *This = impl_from_IBaseFilter(iface);
-    return IUnknown_QueryInterface(This->outerUnknown, riid, ppv);
-}
-
-static ULONG WINAPI AudioRecord_AddRef(IBaseFilter *iface)
-{
-    AudioRecord *This = impl_from_IBaseFilter(iface);
-    return IUnknown_AddRef(This->outerUnknown);
-}
-
-static ULONG WINAPI AudioRecord_Release(IBaseFilter *iface)
-{
-    AudioRecord *This = impl_from_IBaseFilter(iface);
-    return IUnknown_Release(This->outerUnknown);
-}
-
 static HRESULT WINAPI AudioRecord_Stop(IBaseFilter *iface)
 {
     AudioRecord *This = impl_from_IBaseFilter(iface);
@@ -153,9 +78,9 @@ static HRESULT WINAPI AudioRecord_Run(IBaseFilter *iface, REFERENCE_TIME tStart)
 }
 
 static const IBaseFilterVtbl AudioRecordVtbl = {
-    AudioRecord_QueryInterface,
-    AudioRecord_AddRef,
-    AudioRecord_Release,
+    BaseFilterImpl_QueryInterface,
+    BaseFilterImpl_AddRef,
+    BaseFilterImpl_Release,
     BaseFilterImpl_GetClassID,
     AudioRecord_Stop,
     AudioRecord_Pause,
@@ -170,41 +95,56 @@ static const IBaseFilterVtbl AudioRecordVtbl = {
     BaseFilterImpl_QueryVendorInfo
 };
 
-static IPin* WINAPI AudioRecord_GetPin(BaseFilter *iface, int pos)
+static IPin *audio_record_get_pin(struct strmbase_filter *iface, unsigned int index)
 {
-    AudioRecord *This = impl_from_BaseFilter(iface);
-    FIXME("(%p, %d): stub\n", This, pos);
+    FIXME("iface %p, index %u, stub!\n", iface, index);
     return NULL;
 }
 
-static LONG WINAPI AudioRecord_GetPinCount(BaseFilter *iface)
+static void audio_record_destroy(struct strmbase_filter *iface)
 {
-    AudioRecord *This = impl_from_BaseFilter(iface);
-    FIXME("(%p): stub\n", This);
-    return 0;
+    AudioRecord *filter = impl_from_strmbase_filter(iface);
+
+    strmbase_filter_cleanup(&filter->filter);
+    CoTaskMemFree(filter);
 }
 
-static const BaseFilterFuncTable AudioRecordFuncs = {
-    AudioRecord_GetPin,
-    AudioRecord_GetPinCount
+static HRESULT audio_record_query_interface(struct strmbase_filter *iface, REFIID iid, void **out)
+{
+    AudioRecord *filter = impl_from_strmbase_filter(iface);
+
+    if (IsEqualGUID(iid, &IID_IPersistPropertyBag))
+        *out = &filter->IPersistPropertyBag_iface;
+    else
+        return E_NOINTERFACE;
+
+    IUnknown_AddRef((IUnknown *)*out);
+    return S_OK;
+}
+
+static const struct strmbase_filter_ops filter_ops =
+{
+    .filter_get_pin = audio_record_get_pin,
+    .filter_destroy = audio_record_destroy,
+    .filter_query_interface = audio_record_query_interface,
 };
 
 static HRESULT WINAPI PPB_QueryInterface(IPersistPropertyBag *iface, REFIID riid, LPVOID *ppv)
 {
     AudioRecord *This = impl_from_IPersistPropertyBag(iface);
-    return IUnknown_QueryInterface(This->outerUnknown, riid, ppv);
+    return IUnknown_QueryInterface(This->filter.outer_unk, riid, ppv);
 }
 
 static ULONG WINAPI PPB_AddRef(IPersistPropertyBag *iface)
 {
     AudioRecord *This = impl_from_IPersistPropertyBag(iface);
-    return IUnknown_AddRef(This->outerUnknown);
+    return IUnknown_AddRef(This->filter.outer_unk);
 }
 
 static ULONG WINAPI PPB_Release(IPersistPropertyBag *iface)
 {
     AudioRecord *This = impl_from_IPersistPropertyBag(iface);
-    return IUnknown_Release(This->outerUnknown);
+    return IUnknown_Release(This->filter.outer_unk);
 }
 
 static HRESULT WINAPI PPB_GetClassID(IPersistPropertyBag *iface, CLSID *pClassID)
@@ -262,34 +202,20 @@ static const IPersistPropertyBagVtbl PersistPropertyBagVtbl =
 
 IUnknown* WINAPI QCAP_createAudioCaptureFilter(IUnknown *outer, HRESULT *phr)
 {
-    HRESULT hr;
     AudioRecord *This = NULL;
 
     FIXME("(%p, %p): the entire CLSID_AudioRecord implementation is just stubs\n", outer, phr);
 
     This = CoTaskMemAlloc(sizeof(*This));
     if (This == NULL) {
-        hr = E_OUTOFMEMORY;
-        goto end;
-    }
-    memset(This, 0, sizeof(*This));
-    This->IUnknown_iface.lpVtbl = &UnknownVtbl;
-    This->IPersistPropertyBag_iface.lpVtbl = &PersistPropertyBagVtbl;
-    if (outer)
-        This->outerUnknown = outer;
-    else
-        This->outerUnknown = &This->IUnknown_iface;
-
-    hr = BaseFilter_Init(&This->filter, &AudioRecordVtbl, &CLSID_AudioRecord,
-            (DWORD_PTR)(__FILE__ ": AudioRecord.csFilter"), &AudioRecordFuncs);
-
-end:
-    *phr = hr;
-    if (SUCCEEDED(hr)) {
-        return (IUnknown*)&This->filter.IBaseFilter_iface;
-    } else {
-        if (This)
-            IBaseFilter_Release(&This->filter.IBaseFilter_iface);
+        *phr = E_OUTOFMEMORY;
         return NULL;
     }
+    memset(This, 0, sizeof(*This));
+    This->IPersistPropertyBag_iface.lpVtbl = &PersistPropertyBagVtbl;
+
+    strmbase_filter_init(&This->filter, &AudioRecordVtbl, outer, &CLSID_AudioRecord, &filter_ops);
+
+    *phr = S_OK;
+    return &This->filter.IUnknown_inner;
 }
