@@ -38,7 +38,6 @@
 #include "vfwmsgs.h"
 #include "amvideo.h"
 
-#include "wine/unicode.h"
 #include "wine/debug.h"
 
 #include <math.h>
@@ -130,20 +129,14 @@ static inline AVISplitterImpl *impl_from_IBaseFilter(IBaseFilter *iface)
  * != S_OK occurs. This means that any error is fatal to processing.
  */
 
-static HRESULT AVISplitter_SendEndOfFile(AVISplitterImpl *This, DWORD streamnumber)
+static HRESULT AVISplitter_SendEndOfFile(AVISplitterImpl *filter, DWORD index)
 {
-    IPin* ppin = NULL;
-    HRESULT hr;
+    IPin *peer;
 
     TRACE("End of file reached\n");
 
-    hr = IPin_ConnectedTo(This->Parser.ppPins[streamnumber+1], &ppin);
-    if (SUCCEEDED(hr))
-    {
-        hr = IPin_EndOfStream(ppin);
-        IPin_Release(ppin);
-    }
-    TRACE("--> %x\n", hr);
+    if ((peer = filter->Parser.sources[index]->pin.pin.pConnectedTo))
+        IPin_EndOfStream(peer);
 
     /* Force the pullpin thread to stop */
     return S_FALSE;
@@ -297,7 +290,7 @@ static HRESULT AVISplitter_next_request(AVISplitterImpl *This, DWORD streamnumbe
 
 static HRESULT AVISplitter_Receive(AVISplitterImpl *This, IMediaSample *sample, DWORD streamnumber)
 {
-    Parser_OutputPin *pin = unsafe_impl_Parser_OutputPin_from_IPin(This->Parser.ppPins[1+streamnumber]);
+    Parser_OutputPin *pin = This->Parser.sources[streamnumber];
     HRESULT hr;
     LONGLONG start, stop, rtstart, rtstop;
     StreamData *stream = &This->streams[streamnumber];
@@ -571,7 +564,7 @@ static HRESULT AVISplitter_ProcessIndex(AVISplitterImpl *This, AVISTDINDEX **ind
     if (!pIndex)
         return E_OUTOFMEMORY;
 
-    IAsyncReader_SyncRead((impl_PullPin_from_IPin(This->Parser.ppPins[0]))->pReader, qwOffset, cb, (BYTE *)pIndex);
+    IAsyncReader_SyncRead(This->Parser.pInputPin->pReader, qwOffset, cb, (BYTE *)pIndex);
     rest = cb - sizeof(AVISUPERINDEX) + sizeof(RIFFCHUNK) + sizeof(pIndex->aIndex);
 
     TRACE("FOURCC: %s\n", debugstr_an((char *)&pIndex->fcc, 4));
@@ -1292,7 +1285,7 @@ static HRESULT WINAPI AVISplitter_seek(IMediaSeeking *iface)
     EnterCriticalSection(&This->Parser.filter.csFilter);
     for (x = 0; x < This->Parser.cStreams; ++x)
     {
-        Parser_OutputPin *pin = unsafe_impl_Parser_OutputPin_from_IPin(This->Parser.ppPins[1+x]);
+        Parser_OutputPin *pin = This->Parser.sources[x];
         StreamData *stream = This->streams + x;
         LONGLONG wanted_frames;
         DWORD last_keyframe = 0, last_keyframeidx = 0, preroll = 0;
@@ -1411,14 +1404,14 @@ static const IBaseFilterVtbl AVISplitterImpl_Vtbl =
     BaseFilterImpl_QueryVendorInfo,
 };
 
-static void avi_splitter_destroy(BaseFilter *iface)
+static void avi_splitter_destroy(struct strmbase_filter *iface)
 {
     AVISplitterImpl *filter = impl_from_IBaseFilter(&iface->IBaseFilter_iface);
     AVISplitter_Flush(filter);
     Parser_Destroy(&filter->Parser);
 }
 
-static const BaseFilterFuncTable avi_splitter_func_table =
+static const struct strmbase_filter_ops filter_ops =
 {
     .filter_get_pin = parser_get_pin,
     .filter_destroy = avi_splitter_destroy,
@@ -1439,7 +1432,7 @@ HRESULT AVISplitter_create(IUnknown *outer, void **out)
     This->oldindex = NULL;
 
     hr = Parser_Create(&This->Parser, &AVISplitterImpl_Vtbl, outer, &CLSID_AviSplitter,
-            &avi_splitter_func_table, sink_name, AVISplitter_Sample, AVISplitter_QueryAccept,
+            &filter_ops, sink_name, AVISplitter_Sample, AVISplitter_QueryAccept,
             AVISplitter_InputPin_PreConnect, AVISplitter_Flush,
             AVISplitter_Disconnect, AVISplitter_first_request,
             AVISplitter_done_process, NULL, AVISplitter_seek, NULL);
