@@ -33,7 +33,6 @@
 
 #include "winternl.h"
 
-#include "wine/unicode.h"
 #include "wine/debug.h"
 
 #include "parser.h"
@@ -168,7 +167,7 @@ static HRESULT parse_header(BYTE *header, LONGLONG *plen, LONGLONG *pduration)
 
 static HRESULT FillBuffer(MPEGSplitterImpl *This, IMediaSample *pCurrentSample)
 {
-    Parser_OutputPin * pOutputPin = unsafe_impl_Parser_OutputPin_from_IPin(This->Parser.ppPins[1]);
+    Parser_OutputPin *pOutputPin = This->Parser.sources[0];
     LONGLONG length = 0;
     LONGLONG pos = BYTES_FROM_MEDIATIME(This->Parser.pInputPin->rtNext);
     LONGLONG time = This->position, rtstop, rtstart;
@@ -320,14 +319,10 @@ static HRESULT MPEGSplitter_process_sample(LPVOID iface, IMediaSample * pSample,
 
         for (i = 0; i < This->Parser.cStreams; i++)
         {
-            IPin* ppin;
+            IPin *peer;
 
-            hr = IPin_ConnectedTo(This->Parser.ppPins[i+1], &ppin);
-            if (SUCCEEDED(hr))
-            {
-                hr = IPin_EndOfStream(ppin);
-                IPin_Release(ppin);
-            }
+            if ((peer = This->Parser.sources[i]->pin.pin.pConnectedTo))
+                hr = IPin_EndOfStream(peer);
             if (FAILED(hr))
                 WARN("Error sending EndOfStream to pin %u (%x)\n", i, hr);
         }
@@ -841,13 +836,13 @@ static const IAMStreamSelectVtbl AMStreamSelectVtbl =
     AMStreamSelect_Enable
 };
 
-static void mpeg_splitter_destroy(BaseFilter *iface)
+static void mpeg_splitter_destroy(struct strmbase_filter *iface)
 {
     MPEGSplitterImpl *filter = impl_from_IBaseFilter(&iface->IBaseFilter_iface);
     Parser_Destroy(&filter->Parser);
 }
 
-static HRESULT mpeg_splitter_query_interface(BaseFilter *iface, REFIID iid, void **out)
+static HRESULT mpeg_splitter_query_interface(struct strmbase_filter *iface, REFIID iid, void **out)
 {
     MPEGSplitterImpl *filter = impl_from_IBaseFilter(&iface->IBaseFilter_iface);
 
@@ -861,7 +856,7 @@ static HRESULT mpeg_splitter_query_interface(BaseFilter *iface, REFIID iid, void
     return E_NOINTERFACE;
 }
 
-static const BaseFilterFuncTable mpeg_splitter_func_table =
+static const struct strmbase_filter_ops filter_ops =
 {
     .filter_get_pin = parser_get_pin,
     .filter_destroy = mpeg_splitter_destroy,
@@ -882,7 +877,7 @@ HRESULT MPEGSplitter_create(IUnknown *outer, void **out)
 
     ZeroMemory(This, sizeof(MPEGSplitterImpl));
     hr = Parser_Create(&This->Parser, &MPEGSplitter_Vtbl, outer, &CLSID_MPEG1Splitter,
-            &mpeg_splitter_func_table, sink_name, MPEGSplitter_process_sample, MPEGSplitter_query_accept,
+            &filter_ops, sink_name, MPEGSplitter_process_sample, MPEGSplitter_query_accept,
             MPEGSplitter_pre_connect, MPEGSplitter_cleanup, MPEGSplitter_disconnect,
             MPEGSplitter_first_request, NULL, NULL, MPEGSplitter_seek, NULL);
     if (FAILED(hr))
