@@ -25,7 +25,6 @@
 #include "vfwmsgs.h"
 #include "amvideo.h"
 
-#include "wine/unicode.h"
 #include "wine/debug.h"
 
 #include <math.h>
@@ -57,24 +56,24 @@ static inline ParserImpl *impl_from_IBaseFilter( IBaseFilter *iface )
     return CONTAINING_RECORD(iface, ParserImpl, filter.IBaseFilter_iface);
 }
 
-static inline ParserImpl *impl_from_BaseFilter( BaseFilter *iface )
+static inline ParserImpl *impl_from_strmbase_filter(struct strmbase_filter *iface)
 {
     return CONTAINING_RECORD(iface, ParserImpl, filter);
 }
 
-IPin *parser_get_pin(BaseFilter *iface, unsigned int index)
+IPin *parser_get_pin(struct strmbase_filter *iface, unsigned int index)
 {
-    ParserImpl *filter = impl_from_BaseFilter(iface);
+    ParserImpl *filter = impl_from_strmbase_filter(iface);
 
-    if (index > filter->cStreams)
-        return NULL;
-
-    IPin_AddRef(filter->ppPins[index]);
-    return filter->ppPins[index];
+    if (!index)
+        return &filter->pInputPin->pin.IPin_iface;
+    else if (index <= filter->cStreams)
+        return &filter->sources[index - 1]->pin.pin.IPin_iface;
+    return NULL;
 }
 
 HRESULT Parser_Create(ParserImpl *pParser, const IBaseFilterVtbl *vtbl, IUnknown *outer,
-        const CLSID *clsid, const BaseFilterFuncTable *func_table, const WCHAR *sink_name,
+        const CLSID *clsid, const struct strmbase_filter_ops *func_table, const WCHAR *sink_name,
         PFN_PROCESS_SAMPLE fnProcessSample, PFN_QUERY_ACCEPT fnQueryAccept, PFN_PRE_CONNECT fnPreConnect,
         PFN_CLEANUP fnCleanup, PFN_DISCONNECT fnDisconnect, REQUESTPROC fnRequest,
         STOPPROCESSPROC fnDone, SourceSeeking_ChangeStop stop,
@@ -83,13 +82,12 @@ HRESULT Parser_Create(ParserImpl *pParser, const IBaseFilterVtbl *vtbl, IUnknown
     HRESULT hr;
     PIN_INFO piInput;
 
-    strmbase_filter_init(&pParser->filter, vtbl, outer, clsid,
-            (DWORD_PTR)(__FILE__ ": ParserImpl.csFilter"), func_table);
+    strmbase_filter_init(&pParser->filter, vtbl, outer, clsid, func_table);
 
     pParser->fnDisconnect = fnDisconnect;
 
     pParser->cStreams = 0;
-    pParser->ppPins = CoTaskMemAlloc(1 * sizeof(IPin *));
+    pParser->sources = CoTaskMemAlloc(0 * sizeof(IPin *));
 
     /* construct input pin */
     piInput.dir = PINDIR_INPUT;
@@ -111,12 +109,11 @@ HRESULT Parser_Create(ParserImpl *pParser, const IBaseFilterVtbl *vtbl, IUnknown
 
     if (SUCCEEDED(hr))
     {
-        pParser->ppPins[0] = &pParser->pInputPin->pin.IPin_iface;
         pParser->pInputPin->fnPreConnect = fnPreConnect;
     }
     else
     {
-        CoTaskMemFree(pParser->ppPins);
+        CoTaskMemFree(pParser->sources);
         strmbase_filter_cleanup(&pParser->filter);
         CoTaskMemFree(pParser);
     }
@@ -157,7 +154,6 @@ ULONG WINAPI Parser_AddRef(IBaseFilter * iface)
 void Parser_Destroy(ParserImpl *This)
 {
     IPin *connected = NULL;
-    ULONG pinref;
     HRESULT hr;
 
     PullPin_WaitForStateChange(This->pInputPin, INFINITE);
@@ -172,18 +168,10 @@ void Parser_Destroy(ParserImpl *This)
         hr = IPin_Disconnect(&This->pInputPin->pin.IPin_iface);
         assert(hr == S_OK);
     }
-    pinref = IPin_Release(&This->pInputPin->pin.IPin_iface);
-    if (pinref)
-    {
-        /* Valgrind could find this, if I kill it here */
-        ERR("pinref should be null, is %u, destroying anyway\n", pinref);
-        assert((LONG)pinref > 0);
 
-        while (pinref)
-            pinref = IPin_Release(&This->pInputPin->pin.IPin_iface);
-    }
+    PullPin_destroy(This->pInputPin);
 
-    CoTaskMemFree(This->ppPins);
+    CoTaskMemFree(This->sources);
     strmbase_filter_cleanup(&This->filter);
 
     TRACE("Destroying parser\n");
@@ -195,7 +183,7 @@ void Parser_Destroy(ParserImpl *This)
 HRESULT WINAPI Parser_Stop(IBaseFilter * iface)
 {
     ParserImpl *This = impl_from_IBaseFilter(iface);
-    PullPin *pin = impl_PullPin_from_IPin(This->ppPins[0]);
+    PullPin *pin = This->pInputPin;
     ULONG i;
 
     TRACE("%p->()\n", This);
@@ -215,9 +203,9 @@ HRESULT WINAPI Parser_Stop(IBaseFilter * iface)
 
     This->filter.state = State_Stopped;
 
-    for (i = 1; i < (This->cStreams + 1); i++)
+    for (i = 0; i < This->cStreams; ++i)
     {
-        BaseOutputPinImpl_Inactive((BaseOutputPin *)This->ppPins[i]);
+        BaseOutputPinImpl_Inactive(&This->sources[i]->pin);
     }
 
     LeaveCriticalSection(&This->filter.csFilter);
@@ -234,7 +222,7 @@ HRESULT WINAPI Parser_Pause(IBaseFilter * iface)
 {
     HRESULT hr = S_OK;
     ParserImpl *This = impl_from_IBaseFilter(iface);
-    PullPin *pin = impl_PullPin_from_IPin(This->ppPins[0]);
+    PullPin *pin = This->pInputPin;
 
     TRACE("%p->()\n", This);
 
@@ -268,7 +256,7 @@ HRESULT WINAPI Parser_Run(IBaseFilter * iface, REFERENCE_TIME tStart)
 {
     HRESULT hr = S_OK;
     ParserImpl *This = impl_from_IBaseFilter(iface);
-    PullPin *pin = impl_PullPin_from_IPin(This->ppPins[0]);
+    PullPin *pin = This->pInputPin;
 
     ULONG i;
 
@@ -288,9 +276,9 @@ HRESULT WINAPI Parser_Run(IBaseFilter * iface, REFERENCE_TIME tStart)
             return S_OK;
         }
 
-        for (i = 1; i < (This->cStreams + 1); i++)
+        for (i = 0; i < This->cStreams; ++i)
         {
-            hr = BaseOutputPinImpl_Active((BaseOutputPin *)This->ppPins[i]);
+            hr = BaseOutputPinImpl_Active(&This->sources[i]->pin);
             if (SUCCEEDED(hr))
                 hr_any = hr;
         }
@@ -315,7 +303,7 @@ HRESULT WINAPI Parser_Run(IBaseFilter * iface, REFERENCE_TIME tStart)
 HRESULT WINAPI Parser_GetState(IBaseFilter * iface, DWORD dwMilliSecsTimeout, FILTER_STATE *pState)
 {
     ParserImpl *This = impl_from_IBaseFilter(iface);
-    PullPin *pin = impl_PullPin_from_IPin(This->ppPins[0]);
+    PullPin *pin = This->pInputPin;
     HRESULT hr = S_OK;
 
     TRACE("%p->(%d, %p)\n", This, dwMilliSecsTimeout, pState);
@@ -337,7 +325,7 @@ HRESULT WINAPI Parser_GetState(IBaseFilter * iface, DWORD dwMilliSecsTimeout, FI
 HRESULT WINAPI Parser_SetSyncSource(IBaseFilter * iface, IReferenceClock *pClock)
 {
     ParserImpl *This = impl_from_IBaseFilter(iface);
-    PullPin *pin = impl_PullPin_from_IPin(This->ppPins[0]);
+    PullPin *pin = This->pInputPin;
 
     TRACE("%p->(%p)\n", This, pClock);
 
@@ -358,81 +346,67 @@ static const BaseOutputPinFuncTable output_BaseOutputFuncTable = {
     Parser_OutputPin_DecideAllocator,
 };
 
-HRESULT Parser_AddPin(ParserImpl * This, const PIN_INFO * piOutput, ALLOCATOR_PROPERTIES * props, const AM_MEDIA_TYPE * amt)
+HRESULT Parser_AddPin(ParserImpl *filter, const PIN_INFO *pin_info,
+        ALLOCATOR_PROPERTIES *props, const AM_MEDIA_TYPE *mt)
 {
-    IPin ** ppOldPins;
-    HRESULT hr;
+    Parser_OutputPin **old_sources;
+    Parser_OutputPin *object;
 
-    ppOldPins = This->ppPins;
+    if (!(object = CoTaskMemAlloc(sizeof(*object))))
+        return E_OUTOFMEMORY;
 
-    This->ppPins = CoTaskMemAlloc((This->cStreams + 2) * sizeof(IPin *));
-    memcpy(This->ppPins, ppOldPins, (This->cStreams + 1) * sizeof(IPin *));
+    old_sources = filter->sources;
 
-    hr = BaseOutputPin_Construct(&Parser_OutputPin_Vtbl, sizeof(Parser_OutputPin), piOutput, &output_BaseOutputFuncTable, &This->filter.csFilter, This->ppPins + (This->cStreams + 1));
+    filter->sources = CoTaskMemAlloc((filter->cStreams + 1) * sizeof(filter->sources[0]));
+    memcpy(filter->sources, old_sources, filter->cStreams * sizeof(filter->sources[0]));
+    filter->sources[filter->cStreams] = object;
 
-    if (SUCCEEDED(hr))
-    {
-        IPin *pPin = This->ppPins[This->cStreams + 1];
-        Parser_OutputPin *pin = unsafe_impl_Parser_OutputPin_from_IPin(pPin);
-        pin->pmt = CoTaskMemAlloc(sizeof(AM_MEDIA_TYPE));
-        CopyMediaType(pin->pmt, amt);
-        pin->dwSamplesProcessed = 0;
+    strmbase_source_init(&object->pin, &Parser_OutputPin_Vtbl, pin_info,
+            &output_BaseOutputFuncTable, &filter->filter.csFilter);
 
-        pin->pin.pin.pinInfo.pFilter = &This->filter.IBaseFilter_iface;
-        pin->allocProps = *props;
-        This->cStreams++;
-        BaseFilterImpl_IncrementPinVersion(&This->filter);
-        CoTaskMemFree(ppOldPins);
-    }
-    else
-    {
-        CoTaskMemFree(This->ppPins);
-        This->ppPins = ppOldPins;
-        ERR("Failed with error %x\n", hr);
-    }
+    object->pmt = CoTaskMemAlloc(sizeof(AM_MEDIA_TYPE));
+    CopyMediaType(object->pmt, mt);
+    object->dwSamplesProcessed = 0;
 
-    return hr;
+    object->pin.pin.pinInfo.pFilter = &filter->filter.IBaseFilter_iface;
+    object->allocProps = *props;
+    filter->cStreams++;
+    BaseFilterImpl_IncrementPinVersion(&filter->filter);
+    CoTaskMemFree(old_sources);
+
+    return S_OK;
 }
 
-static HRESULT WINAPI break_connection(IPin *iface)
+static void free_source_pin(Parser_OutputPin *pin)
 {
-    Parser_OutputPin *pin = unsafe_impl_Parser_OutputPin_from_IPin(iface);
-    HRESULT hr;
-
-    if (!pin->pin.pin.pConnectedTo || !pin->pin.pMemInputPin)
-        hr = VFW_E_NOT_CONNECTED;
-    else
+    if (pin->pin.pin.pConnectedTo)
     {
-        hr = IPin_Disconnect(pin->pin.pin.pConnectedTo);
-        IPin_Disconnect(iface);
+        IPin_Disconnect(pin->pin.pin.pConnectedTo);
+        IPin_Disconnect(&pin->pin.pin.IPin_iface);
     }
 
-    return hr;
+    FreeMediaType(pin->pmt);
+    CoTaskMemFree(pin->pmt);
+    strmbase_source_cleanup(&pin->pin);
+    CoTaskMemFree(pin);
 }
 
 static HRESULT Parser_RemoveOutputPins(ParserImpl * This)
 {
     /* NOTE: should be in critical section when calling this function */
-    HRESULT hr;
     ULONG i;
-    IPin ** ppOldPins = This->ppPins;
+    Parser_OutputPin **old_sources = This->sources;
 
     TRACE("(%p)\n", This);
 
-    /* reduce the pin array down to 1 (just our input pin) */
-    This->ppPins = CoTaskMemAlloc(sizeof(IPin *) * 1);
-    memcpy(This->ppPins, ppOldPins, sizeof(IPin *) * 1);
+    This->sources = CoTaskMemAlloc(0);
 
     for (i = 0; i < This->cStreams; i++)
-    {
-        hr = break_connection(ppOldPins[i + 1]);
-        TRACE("Disconnect: %08x\n", hr);
-        IPin_Release(ppOldPins[i + 1]);
-    }
+        free_source_pin(old_sources[i]);
 
     BaseFilterImpl_IncrementPinVersion(&This->filter);
     This->cStreams = 0;
-    CoTaskMemFree(ppOldPins);
+    CoTaskMemFree(old_sources);
 
     return S_OK;
 }
@@ -578,26 +552,6 @@ static HRESULT WINAPI Parser_OutputPin_QueryInterface(IPin * iface, REFIID riid,
     return E_NOINTERFACE;
 }
 
-static ULONG WINAPI Parser_OutputPin_Release(IPin * iface)
-{
-    Parser_OutputPin *This = unsafe_impl_Parser_OutputPin_from_IPin(iface);
-    ULONG refCount = InterlockedDecrement(&This->pin.pin.refCount);
-    
-    TRACE("(%p)->() Release from %d\n", iface, refCount + 1);
-
-    if (!refCount)
-    {
-        FreeMediaType(This->pmt);
-        CoTaskMemFree(This->pmt);
-        FreeMediaType(&This->pin.pin.mtCurrent);
-        if (This->pin.pAllocator)
-            IMemAllocator_Release(This->pin.pAllocator);
-        CoTaskMemFree(This);
-        return 0;
-    }
-    return refCount;
-}
-
 static HRESULT WINAPI Parser_OutputPin_Connect(IPin * iface, IPin * pReceivePin, const AM_MEDIA_TYPE * pmt)
 {
     Parser_OutputPin *This = unsafe_impl_Parser_OutputPin_from_IPin(iface);
@@ -624,7 +578,7 @@ static const IPinVtbl Parser_OutputPin_Vtbl =
 {
     Parser_OutputPin_QueryInterface,
     BasePinImpl_AddRef,
-    Parser_OutputPin_Release,
+    BasePinImpl_Release,
     Parser_OutputPin_Connect,
     BaseOutputPinImpl_ReceiveConnection,
     BaseOutputPinImpl_Disconnect,
@@ -737,7 +691,7 @@ static const IPinVtbl Parser_InputPin_Vtbl =
 {
     Parser_PullPin_QueryInterface,
     BasePinImpl_AddRef,
-    PullPin_Release,
+    BasePinImpl_Release,
     BaseInputPinImpl_Connect,
     Parser_PullPin_ReceiveConnection,
     Parser_PullPin_Disconnect,
