@@ -193,7 +193,9 @@ BOOL WINAPI BaseControlWindowImpl_PossiblyEatMessage(BaseWindow *This, UINT uMsg
     return FALSE;
 }
 
-HRESULT WINAPI BaseControlWindow_Init(BaseControlWindow *pControlWindow, const IVideoWindowVtbl *lpVtbl, BaseFilter *owner, CRITICAL_SECTION *lock, BasePin* pPin,const BaseWindowFuncTable *pFuncsTable)
+HRESULT WINAPI BaseControlWindow_Init(BaseControlWindow *pControlWindow,
+        const IVideoWindowVtbl *lpVtbl, struct strmbase_filter *owner,
+        CRITICAL_SECTION *lock, BasePin* pPin,const BaseWindowFuncTable *pFuncsTable)
 {
     HRESULT hr;
 
@@ -217,6 +219,24 @@ HRESULT WINAPI BaseControlWindow_Destroy(BaseControlWindow *pControlWindow)
     return S_OK;
 }
 
+HRESULT WINAPI BaseControlWindowImpl_QueryInterface(IVideoWindow *iface, REFIID iid, void **out)
+{
+    BaseControlWindow *window = impl_from_IVideoWindow(iface);
+    return IUnknown_QueryInterface(window->pFilter->outer_unk, iid, out);
+}
+
+ULONG WINAPI BaseControlWindowImpl_AddRef(IVideoWindow *iface)
+{
+    BaseControlWindow *window = impl_from_IVideoWindow(iface);
+    return IUnknown_AddRef(window->pFilter->outer_unk);
+}
+
+ULONG WINAPI BaseControlWindowImpl_Release(IVideoWindow *iface)
+{
+    BaseControlWindow *window = impl_from_IVideoWindow(iface);
+    return IUnknown_Release(window->pFilter->outer_unk);
+}
+
 HRESULT WINAPI BaseControlWindowImpl_GetTypeInfoCount(IVideoWindow *iface, UINT *count)
 {
     TRACE("iface %p, count %p.\n", iface, count);
