@@ -47,7 +47,7 @@ static BOOL get_func_id(vbdisp_t *This, const WCHAR *name, vbdisp_invoke_type_t
                 continue;
         }
 
-        if(!strcmpiW(This->desc->funcs[i].name, name)) {
+        if(!wcsicmp(This->desc->funcs[i].name, name)) {
             *id = i;
             return TRUE;
         }
@@ -67,7 +67,7 @@ HRESULT vbdisp_get_id(vbdisp_t *This, BSTR name, vbdisp_invoke_type_t invoke_typ
         if(!search_private && !This->desc->props[i].is_public)
             continue;
 
-        if(!strcmpiW(This->desc->props[i].name, name)) {
+        if(!wcsicmp(This->desc->props[i].name, name)) {
             *id = i + This->desc->func_cnt;
             return S_OK;
         }
@@ -232,11 +232,11 @@ static HRESULT invoke_builtin(vbdisp_t *This, const builtin_prop_t *prop, WORD f
     argn = arg_cnt(dp);
 
     if(argn < prop->min_args || argn > (prop->max_args ? prop->max_args : prop->min_args)) {
-        FIXME("invalid number of arguments\n");
-        return E_FAIL;
+        WARN("invalid number of arguments\n");
+        return MAKE_VBSERROR(VBSE_FUNC_ARITY_MISMATCH);
     }
 
-    assert(argn < sizeof(args)/sizeof(*args));
+    assert(argn < ARRAY_SIZE(args));
 
     for(i=0; i < argn; i++) {
         if(V_VT(dp->rgvarg+dp->cArgs-i-1) == (VT_BYREF|VT_VARIANT))
@@ -647,7 +647,7 @@ HRESULT create_procedure_disp(script_ctx_t *ctx, vbscode_t *code, IDispatch **re
         return E_OUTOFMEMORY;
 
     desc->ctx = ctx;
-    desc->builtin_prop_cnt = sizeof(procedure_props)/sizeof(*procedure_props);
+    desc->builtin_prop_cnt = ARRAY_SIZE(procedure_props);
     desc->builtin_props = procedure_props;
     desc->value_func = &code->main_code;
 
@@ -823,14 +823,14 @@ static HRESULT WINAPI ScriptDisp_GetDispID(IDispatchEx *iface, BSTR bstrName, DW
         return E_UNEXPECTED;
 
     for(ident = This->ident_map; ident < This->ident_map+This->ident_map_cnt; ident++) {
-        if(!strcmpiW(ident->name, bstrName)) {
+        if(!wcsicmp(ident->name, bstrName)) {
             *pid = ident_to_id(This, ident);
             return S_OK;
         }
     }
 
     for(var = This->ctx->global_vars; var; var = var->next) {
-        if(!strcmpiW(var->name, bstrName)) {
+        if(!wcsicmp(var->name, bstrName)) {
             ident = add_ident(This, var->name);
             if(!ident)
                 return E_OUTOFMEMORY;
@@ -843,7 +843,7 @@ static HRESULT WINAPI ScriptDisp_GetDispID(IDispatchEx *iface, BSTR bstrName, DW
     }
 
     for(func = This->ctx->global_funcs; func; func = func->next) {
-        if(!strcmpiW(func->name, bstrName)) {
+        if(!wcsicmp(func->name, bstrName)) {
             ident = add_ident(This, func->name);
             if(!ident)
                 return E_OUTOFMEMORY;
