@@ -215,6 +215,7 @@ HRESULT WINAPI WsCreateServiceProxyFromTemplate( WS_CHANNEL_TYPE channel_type,
 HRESULT WINAPI WsResetServiceProxy( WS_SERVICE_PROXY *handle, WS_ERROR *error )
 {
     struct proxy *proxy = (struct proxy *)handle;
+    HRESULT hr = S_OK;
 
     TRACE( "%p %p\n", handle, error );
     if (error) FIXME( "ignoring error parameter\n" );
@@ -230,15 +231,13 @@ HRESULT WINAPI WsResetServiceProxy( WS_SERVICE_PROXY *handle, WS_ERROR *error )
     }
 
     if (proxy->state != WS_SERVICE_PROXY_STATE_CREATED && proxy->state != WS_SERVICE_PROXY_STATE_CLOSED)
-    {
-        LeaveCriticalSection( &proxy->cs );
-        return WS_E_INVALID_OPERATION;
-    }
-
-    reset_proxy( proxy );
+        hr = WS_E_INVALID_OPERATION;
+    else
+        reset_proxy( proxy );
 
     LeaveCriticalSection( &proxy->cs );
-    return S_OK;
+    TRACE( "returning %08x\n", hr );
+    return hr;
 }
 
 /**************************************************************************
@@ -300,6 +299,7 @@ HRESULT WINAPI WsGetServiceProxyProperty( WS_SERVICE_PROXY *handle, WS_PROXY_PRO
     }
 
     LeaveCriticalSection( &proxy->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -330,6 +330,7 @@ HRESULT WINAPI WsOpenServiceProxy( WS_SERVICE_PROXY *handle, const WS_ENDPOINT_A
         proxy->state = WS_SERVICE_PROXY_STATE_OPEN;
 
     LeaveCriticalSection( &proxy->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -359,6 +360,7 @@ HRESULT WINAPI WsCloseServiceProxy( WS_SERVICE_PROXY *handle, const WS_ASYNC_CON
         proxy->state = WS_SERVICE_PROXY_STATE_CLOSED;
 
     LeaveCriticalSection( &proxy->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -540,5 +542,6 @@ HRESULT WINAPI WsCall( WS_SERVICE_PROXY *handle, const WS_OPERATION_DESCRIPTION
 done:
     WsFreeMessage( msg );
     LeaveCriticalSection( &proxy->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
