@@ -530,7 +530,7 @@ static void wined3d_cs_exec_present(struct wined3d_cs *cs, const void *data)
     swapchain->swapchain_ops->swapchain_present(swapchain, &op->src_rect, &op->dst_rect, op->swap_interval, op->flags);
 
     wined3d_resource_release(&swapchain->front_buffer->resource);
-    for (i = 0; i < swapchain->desc.backbuffer_count; ++i)
+    for (i = 0; i < swapchain->state.desc.backbuffer_count; ++i)
     {
         wined3d_resource_release(&swapchain->back_buffers[i]->resource);
     }
@@ -558,7 +558,7 @@ void wined3d_cs_emit_present(struct wined3d_cs *cs, struct wined3d_swapchain *sw
     pending = InterlockedIncrement(&cs->pending_presents);
 
     wined3d_resource_acquire(&swapchain->front_buffer->resource);
-    for (i = 0; i < swapchain->desc.backbuffer_count; ++i)
+    for (i = 0; i < swapchain->state.desc.backbuffer_count; ++i)
     {
         wined3d_resource_acquire(&swapchain->back_buffers[i]->resource);
     }
@@ -1008,12 +1008,11 @@ void wined3d_cs_emit_draw_indirect(struct wined3d_cs *cs, GLenum primitive_type,
 
 static void wined3d_cs_exec_flush(struct wined3d_cs *cs, const void *data)
 {
-    struct wined3d_context_gl *context_gl;
+    struct wined3d_context *context;
 
-    context_gl = wined3d_context_gl(context_acquire(cs->device, NULL, 0));
-    if (context_gl->valid)
-        context_gl->c.gl_info->gl_ops.gl.p_glFlush();
-    context_release(&context_gl->c);
+    context = context_acquire(cs->device, NULL, 0);
+    cs->device->adapter->adapter_ops->adapter_flush_context(context);
+    context_release(context);
 }
 
 void wined3d_cs_emit_flush(struct wined3d_cs *cs)
@@ -1137,7 +1136,7 @@ static void wined3d_cs_exec_set_depth_stencil_view(struct wined3d_cs *cs, const
     {
         struct wined3d_texture *prev_texture = texture_from_resource(prev->resource);
 
-        if (device->swapchains[0]->desc.flags & WINED3D_SWAPCHAIN_DISCARD_DEPTHSTENCIL
+        if (device->swapchains[0]->state.desc.flags & WINED3D_SWAPCHAIN_DISCARD_DEPTHSTENCIL
                 || prev_texture->flags & WINED3D_TEXTURE_DISCARD)
             wined3d_texture_validate_location(prev_texture,
                     prev->sub_resource_idx, WINED3D_LOCATION_DISCARDED);
@@ -1153,9 +1152,12 @@ static void wined3d_cs_exec_set_depth_stencil_view(struct wined3d_cs *cs, const
         device_invalidate_state(device, STATE_RENDER(WINED3D_RS_STENCILWRITEMASK));
         device_invalidate_state(device, STATE_RENDER(WINED3D_RS_DEPTHBIAS));
     }
-    else if (prev && prev->format->depth_bias_scale != op->view->format->depth_bias_scale)
+    else if (prev)
     {
-        device_invalidate_state(device, STATE_RENDER(WINED3D_RS_DEPTHBIAS));
+        if (prev->format->depth_bias_scale != op->view->format->depth_bias_scale)
+            device_invalidate_state(device, STATE_RENDER(WINED3D_RS_DEPTHBIAS));
+        if (prev->format->stencil_size != op->view->format->stencil_size)
+            device_invalidate_state(device, STATE_RENDER(WINED3D_RS_STENCILREF));
     }
 
     device_invalidate_state(device, STATE_FRAMEBUFFER);
@@ -2228,7 +2230,8 @@ static void wined3d_cs_exec_blt_sub_resource(struct wined3d_cs *cs, const void *
                 && update_h == wined3d_texture_get_level_height(dst_texture, level)
                 && update_d == wined3d_texture_get_level_depth(dst_texture, level))
         {
-            wined3d_texture_prepare_texture(dst_texture, context, FALSE);
+            wined3d_texture_prepare_location(dst_texture, op->dst_sub_resource_idx,
+                    context, WINED3D_LOCATION_TEXTURE_RGB);
         }
         else if (!wined3d_texture_load_location(dst_texture, op->dst_sub_resource_idx,
                 context, WINED3D_LOCATION_TEXTURE_RGB))
@@ -2337,14 +2340,14 @@ static void wined3d_cs_exec_update_sub_resource(struct wined3d_cs *cs, const voi
     /* Only load the sub-resource for partial updates. */
     if (!box->left && !box->top && !box->front
             && box->right == width && box->bottom == height && box->back == depth)
-        wined3d_texture_prepare_texture(texture, context, FALSE);
+        wined3d_texture_prepare_location(texture, op->sub_resource_idx, context, WINED3D_LOCATION_TEXTURE_RGB);
     else
         wined3d_texture_load_location(texture, op->sub_resource_idx, context, WINED3D_LOCATION_TEXTURE_RGB);
-    wined3d_texture_gl_bind_and_dirtify(wined3d_texture_gl(texture), wined3d_context_gl(context), FALSE);
 
     wined3d_box_set(&src_box, 0, 0, box->right - box->left, box->bottom - box->top, 0, box->back - box->front);
-    wined3d_texture_upload_data(texture, op->sub_resource_idx, context, texture->resource.format, &src_box,
-            &addr, op->data.row_pitch, op->data.slice_pitch, box->left, box->top, box->front, FALSE);
+    texture->texture_ops->texture_upload_data(context, &addr, texture->resource.format, &src_box,
+            op->data.row_pitch, op->data.slice_pitch, texture, op->sub_resource_idx,
+            WINED3D_LOCATION_TEXTURE_RGB, box->left, box->top, box->front);
 
     wined3d_texture_validate_location(texture, op->sub_resource_idx, WINED3D_LOCATION_TEXTURE_RGB);
     wined3d_texture_invalidate_location(texture, op->sub_resource_idx, ~WINED3D_LOCATION_TEXTURE_RGB);
