@@ -217,7 +217,7 @@ static HRESULT vertexdeclaration_init(struct wined3d_vertex_declaration *declara
 
         /* Find the streams used in the declaration. The vertex buffers have
          * to be loaded when drawing, but filter tesselation pseudo streams. */
-        if (e->input_slot >= MAX_STREAMS)
+        if (e->input_slot >= WINED3D_MAX_STREAMS)
             continue;
 
         if (!(e->format->flags[WINED3D_GL_RES_TYPE_BUFFER] & WINED3DFMT_FLAG_VERTEX_ATTRIBUTE))
@@ -251,11 +251,6 @@ static HRESULT vertexdeclaration_init(struct wined3d_vertex_declaration *declara
             heap_free(declaration->elements);
             return E_FAIL;
         }
-
-        if (elements[i].format == WINED3DFMT_R16G16_FLOAT || elements[i].format == WINED3DFMT_R16G16B16A16_FLOAT)
-        {
-            declaration->have_half_floats = TRUE;
-        }
     }
 
     return WINED3D_OK;
