@@ -749,6 +749,14 @@ static const char * const focus_details[] =
     "NotifyDetailNone"
 };
 
+static const char * const focus_modes[] =
+{
+    "NotifyNormal",
+    "NotifyGrab",
+    "NotifyUngrab",
+    "NotifyWhileGrabbed"
+};
+
 /**********************************************************************
  *              X11DRV_FocusIn
  */
@@ -759,7 +767,7 @@ static BOOL X11DRV_FocusIn( HWND hwnd, XEvent *xev )
 
     if (!hwnd) return FALSE;
 
-    TRACE( "win %p xwin %lx detail=%s\n", hwnd, event->window, focus_details[event->detail] );
+    TRACE( "win %p xwin %lx detail=%s mode=%s\n", hwnd, event->window, focus_details[event->detail], focus_modes[event->mode] );
 
     if (event->detail == NotifyPointer) return FALSE;
     if (hwnd == GetDesktopWindow()) return FALSE;
@@ -786,7 +794,7 @@ static BOOL X11DRV_FocusIn( HWND hwnd, XEvent *xev )
 /**********************************************************************
  *              focus_out
  */
- static void focus_out( Display *display , HWND hwnd )
+static void focus_out( Display *display , HWND hwnd )
  {
     HWND hwnd_tmp;
     Window focus_win;
@@ -839,7 +847,7 @@ static BOOL X11DRV_FocusOut( HWND hwnd, XEvent *xev )
 {
     XFocusChangeEvent *event = &xev->xfocus;
 
-    TRACE( "win %p xwin %lx detail=%s\n", hwnd, event->window, focus_details[event->detail] );
+    TRACE( "win %p xwin %lx detail=%s mode=%s\n", hwnd, event->window, focus_details[event->detail], focus_modes[event->mode] );
 
     if (event->detail == NotifyPointer)
     {
@@ -858,7 +866,7 @@ static BOOL X11DRV_FocusOut( HWND hwnd, XEvent *xev )
 static BOOL X11DRV_Expose( HWND hwnd, XEvent *xev )
 {
     XExposeEvent *event = &xev->xexpose;
-    RECT rect;
+    RECT rect, abs_rect;
     POINT pos;
     struct x11drv_win_data *data;
     HRGN surface_region = 0;
@@ -901,14 +909,16 @@ static BOOL X11DRV_Expose( HWND hwnd, XEvent *xev )
     {
         if (GetWindowLongW( data->hwnd, GWL_EXSTYLE ) & WS_EX_LAYOUTRTL)
             mirror_rect( &data->client_rect, &rect );
+        abs_rect = rect;
+        MapWindowPoints( hwnd, 0, (POINT *)&abs_rect, 2 );
 
         SERVER_START_REQ( update_window_zorder )
         {
             req->window      = wine_server_user_handle( hwnd );
-            req->rect.left   = rect.left;
-            req->rect.top    = rect.top;
-            req->rect.right  = rect.right;
-            req->rect.bottom = rect.bottom;
+            req->rect.left   = abs_rect.left;
+            req->rect.top    = abs_rect.top;
+            req->rect.right  = abs_rect.right;
+            req->rect.bottom = abs_rect.bottom;
             wine_server_call( req );
         }
         SERVER_END_REQ;
@@ -1077,17 +1087,13 @@ static BOOL X11DRV_ConfigureNotify( HWND hwnd, XEvent *xev )
     }
     else pos = root_to_virtual_screen( x, y );
 
-    rect.left   = pos.x;
-    rect.top    = pos.y;
-    rect.right  = pos.x + event->width;
-    rect.bottom = pos.y + event->height;
+    X11DRV_X_to_window_rect( data, &rect, pos.x, pos.y, event->width, event->height );
+    if (root_coords) MapWindowPoints( 0, parent, (POINT *)&rect, 2 );
+
     TRACE( "win %p/%lx new X rect %d,%d,%dx%d (event %d,%d,%dx%d)\n",
            hwnd, data->whole_window, rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top,
            event->x, event->y, event->width, event->height );
 
-    X11DRV_X_to_window_rect( data, &rect );
-    if (root_coords) MapWindowPoints( 0, parent, (POINT *)&rect, 2 );
-
     /* Compare what has changed */
 
     x     = rect.left;
@@ -1155,7 +1161,8 @@ static BOOL X11DRV_GravityNotify( HWND hwnd, XEvent *xev )
 {
     XGravityEvent *event = &xev->xgravity;
     struct x11drv_win_data *data = get_win_data( hwnd );
-    RECT rect, window_rect;
+    RECT window_rect;
+    int x, y;
 
     if (!data) return FALSE;
 
@@ -1165,22 +1172,18 @@ static BOOL X11DRV_GravityNotify( HWND hwnd, XEvent *xev )
         return FALSE;
     }
 
-    rect.left   = event->x;
-    rect.top    = event->y;
-    rect.right  = rect.left + data->whole_rect.right - data->whole_rect.left;
-    rect.bottom = rect.top + data->whole_rect.bottom - data->whole_rect.top;
+    x = event->x + data->window_rect.left - data->whole_rect.left;
+    y = event->y + data->window_rect.top - data->whole_rect.top;
 
-    TRACE( "win %p/%lx new X rect %d,%d,%dx%d (event %d,%d)\n",
-           hwnd, data->whole_window, rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top,
-           event->x, event->y );
+    TRACE( "win %p/%lx new X pos %d,%d (event %d,%d)\n",
+           hwnd, data->whole_window, x, y, event->x, event->y );
 
-    X11DRV_X_to_window_rect( data, &rect );
     window_rect = data->window_rect;
     release_win_data( data );
 
-    if (window_rect.left != rect.left || window_rect.top != rect.top)
-        SetWindowPos( hwnd, 0, rect.left, rect.top, 0, 0,
-                      SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE | SWP_NOCOPYBITS );
+    if (window_rect.left != x || window_rect.top != y)
+        SetWindowPos( hwnd, 0, x, y, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE | SWP_NOCOPYBITS );
+
     return TRUE;
 }
 
@@ -1271,6 +1274,8 @@ static void handle_wm_state_notify( HWND hwnd, XPropertyEvent *event, BOOL updat
             {
                 TRACE( "restoring win %p/%lx\n", data->hwnd, data->whole_window );
                 release_win_data( data );
+                if ((style & (WS_MINIMIZE | WS_VISIBLE)) == (WS_MINIMIZE | WS_VISIBLE))
+                    SetActiveWindow( hwnd );
                 SendMessageW( hwnd, WM_SYSCOMMAND, SC_RESTORE, 0 );
                 return;
             }
@@ -1627,8 +1632,8 @@ static void EVENT_DropURLs( HWND hWnd, XClientMessageEvent *event )
         PostMessageA( hWnd, WM_DROPFILES, (WPARAM)hDrop, 0L );
       }
     }
-    if( p_data ) XFree(p_data);
   }
+  if( p_data ) XFree(p_data);
 }
 
 
@@ -1744,7 +1749,7 @@ static BOOL X11DRV_ClientMessage( HWND hwnd, XEvent *xev )
         return FALSE;
     }
 
-    for (i = 0; i < sizeof(client_messages)/sizeof(client_messages[0]); i++)
+    for (i = 0; i < ARRAY_SIZE( client_messages ); i++)
     {
         if (event->message_type == X11DRV_Atoms[client_messages[i].atom - FIRST_XATOM])
         {
