@@ -58,6 +58,7 @@ MAKE_FUNCPTR(XRRGetCrtcInfo)
 MAKE_FUNCPTR(XRRGetOutputInfo)
 MAKE_FUNCPTR(XRRGetScreenResources)
 MAKE_FUNCPTR(XRRSetCrtcConfig)
+MAKE_FUNCPTR(XRRSetScreenSize)
 static typeof(XRRGetScreenResources) *pXRRGetScreenResourcesCurrent;
 static RRMode *xrandr12_modes;
 static int primary_crtc;
@@ -102,6 +103,7 @@ static int load_xrandr(void)
         LOAD_FUNCPTR(XRRGetOutputInfo)
         LOAD_FUNCPTR(XRRGetScreenResources)
         LOAD_FUNCPTR(XRRSetCrtcConfig)
+        LOAD_FUNCPTR(XRRSetScreenSize)
         r = 2;
 #endif
 #undef LOAD_FUNCPTR
@@ -272,6 +274,20 @@ static void xrandr10_init_modes(void)
 
 #ifdef HAVE_XRRGETSCREENRESOURCES
 
+static XRRScreenResources *xrandr_get_screen_resources(void)
+{
+    XRRScreenResources *resources = pXRRGetScreenResourcesCurrent( gdi_display, root_window );
+    if (resources && !resources->ncrtc)
+    {
+        pXRRFreeScreenResources( resources );
+        resources = pXRRGetScreenResources( gdi_display, root_window );
+    }
+
+    if (!resources)
+        ERR("Failed to get screen resources.\n");
+    return resources;
+}
+
 static int xrandr12_get_current_mode(void)
 {
     XRRScreenResources *resources;
@@ -320,8 +336,30 @@ static int xrandr12_get_current_mode(void)
     return ret;
 }
 
+static void get_screen_size( XRRScreenResources *resources, unsigned int *width, unsigned int *height )
+{
+    XRRCrtcInfo *crtc_info;
+    int i;
+    *width = *height = 0;
+
+    for (i = 0; i < resources->ncrtc; ++i)
+    {
+        if (!(crtc_info = pXRRGetCrtcInfo( gdi_display, resources, resources->crtcs[i] )))
+            continue;
+
+        if (crtc_info->mode != None)
+        {
+            *width = max(*width, crtc_info->x + crtc_info->width);
+            *height = max(*height, crtc_info->y + crtc_info->height);
+        }
+
+        pXRRFreeCrtcInfo( crtc_info );
+    }
+}
+
 static LONG xrandr12_set_current_mode( int mode )
 {
+    unsigned int screen_width, screen_height;
     Status status = RRSetConfigFailed;
     XRRScreenResources *resources;
     XRRCrtcInfo *crtc_info;
@@ -345,10 +383,41 @@ static LONG xrandr12_set_current_mode( int mode )
     TRACE("CRTC %d: mode %#lx, %ux%u+%d+%d.\n", primary_crtc, crtc_info->mode,
           crtc_info->width, crtc_info->height, crtc_info->x, crtc_info->y);
 
+    /* According to the RandR spec, the entire CRTC must fit inside the screen.
+     * Since we use the union of all enabled CRTCs to determine the necessary
+     * screen size, this might involve shrinking the screen, so we must disable
+     * the CRTC in question first. */
+
+    XGrabServer( gdi_display );
+
+    status = pXRRSetCrtcConfig( gdi_display, resources, resources->crtcs[primary_crtc],
+                                CurrentTime, crtc_info->x, crtc_info->y, None,
+                                crtc_info->rotation, NULL, 0 );
+    if (status != RRSetConfigSuccess)
+    {
+        XUngrabServer( gdi_display );
+        ERR("Failed to disable CRTC.\n");
+        pXRRFreeCrtcInfo( crtc_info );
+        pXRRFreeScreenResources( resources );
+        return DISP_CHANGE_FAILED;
+    }
+
+    get_screen_size( resources, &screen_width, &screen_height );
+    screen_width = max( screen_width, crtc_info->x + dd_modes[mode].width );
+    screen_height = max( screen_height, crtc_info->y + dd_modes[mode].height );
+
+    pXRRSetScreenSize( gdi_display, root_window, screen_width, screen_height,
+            screen_width * DisplayWidthMM( gdi_display, default_visual.screen )
+                         / DisplayWidth( gdi_display, default_visual.screen ),
+            screen_height * DisplayHeightMM( gdi_display, default_visual.screen )
+                         / DisplayHeight( gdi_display, default_visual.screen ));
+
     status = pXRRSetCrtcConfig( gdi_display, resources, resources->crtcs[primary_crtc],
                                 CurrentTime, crtc_info->x, crtc_info->y, xrandr12_modes[mode],
                                 crtc_info->rotation, crtc_info->outputs, crtc_info->noutput );
 
+    XUngrabServer( gdi_display );
+
     pXRRFreeCrtcInfo( crtc_info );
     pXRRFreeScreenResources( resources );
 
@@ -393,21 +462,8 @@ static int xrandr12_init_modes(void)
     int ret = -1;
     int i, j;
 
-    if (!(resources = pXRRGetScreenResourcesCurrent( gdi_display, root_window )))
-    {
-        ERR("Failed to get screen resources.\n");
+    if (!(resources = xrandr_get_screen_resources()))
         return ret;
-    }
-
-    if (!resources->ncrtc)
-    {
-        pXRRFreeScreenResources( resources );
-        if (!(resources = pXRRGetScreenResources( gdi_display, root_window )))
-        {
-            ERR("Failed to get screen resources.\n");
-            return ret;
-        }
-    }
 
     if (!(crtc_info = xrandr12_get_primary_crtc_info( resources, &primary_crtc )))
     {
