@@ -15,7 +15,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
 #include <stdarg.h>
 #include <stdlib.h>
 #include "windef.h"
@@ -71,6 +70,27 @@ BOOL WINAPI WTSEnableChildSessions(BOOL enable)
     return TRUE;
 }
 
+
+/************************************************************
+ *                WTSEnumerateProcessesExW  (WTSAPI32.@)
+ */
+BOOL WINAPI WTSEnumerateProcessesExW(HANDLE server, DWORD *level, DWORD session_id, WCHAR **info, DWORD *count)
+{
+    FIXME("Stub %p %p %d %p %p\n", server, level, session_id, info, count);
+    if (count) *count = 0;
+    return FALSE;
+}
+
+/************************************************************
+ *                WTSEnumerateProcessesExA  (WTSAPI32.@)
+ */
+BOOL WINAPI WTSEnumerateProcessesExA(HANDLE server, DWORD *level, DWORD session_id, char **info, DWORD *count)
+{
+    FIXME("Stub %p %p %d %p %p\n", server, level, session_id, info, count);
+    if (count) *count = 0;
+    return FALSE;
+}
+
 /************************************************************
  *                WTSEnumerateProcessesA  (WTSAPI32.@)
  */
@@ -128,6 +148,26 @@ BOOL WINAPI WTSEnumerateServersW(LPWSTR pDomainName, DWORD Reserved, DWORD Versi
 }
 
 
+/************************************************************
+ *                WTSEnumerateEnumerateSessionsExW  (WTSAPI32.@)
+ */
+BOOL WINAPI WTSEnumerateSessionsExW(HANDLE server, DWORD *level, DWORD filter, WTS_SESSION_INFO_1W* info, DWORD *count)
+{
+    FIXME("Stub %p %p %d %p %p\n", server, level, filter, info, count);
+    if (count) *count = 0;
+    return FALSE;
+}
+
+/************************************************************
+ *                WTSEnumerateEnumerateSessionsExA  (WTSAPI32.@)
+ */
+BOOL WINAPI WTSEnumerateSessionsExA(HANDLE server, DWORD *level, DWORD filter, WTS_SESSION_INFO_1A* info, DWORD *count)
+{
+    FIXME("Stub %p %p %d %p %p\n", server, level, filter, info, count);
+    if (count) *count = 0;
+    return FALSE;
+}
+
 /************************************************************
  *                WTSEnumerateEnumerateSessionsA  (WTSAPI32.@)
  */
@@ -172,6 +212,27 @@ void WINAPI WTSFreeMemory(PVOID pMemory)
     heap_free(pMemory);
 }
 
+/************************************************************
+ *                WTSFreeMemoryExA (WTSAPI32.@)
+ */
+BOOL WINAPI WTSFreeMemoryExA(WTS_TYPE_CLASS type, void *ptr, ULONG nmemb)
+{
+    TRACE("%d %p %d\n", type, ptr, nmemb);
+    heap_free(ptr);
+    return TRUE;
+}
+
+/************************************************************
+ *                WTSFreeMemoryExW (WTSAPI32.@)
+ */
+BOOL WINAPI WTSFreeMemoryExW(WTS_TYPE_CLASS type, void *ptr, ULONG nmemb)
+{
+    TRACE("%d %p %d\n", type, ptr, nmemb);
+    heap_free(ptr);
+    return TRUE;
+}
+
+
 /************************************************************
  *                WTSLogoffSession (WTSAPI32.@)
  */
@@ -182,6 +243,27 @@ BOOL WINAPI WTSLogoffSession(HANDLE hserver, DWORD session_id, BOOL bwait)
     return FALSE;
 }
 
+
+/************************************************************
+ *                WTSOpenServerExW (WTSAPI32.@)
+ */
+HANDLE WINAPI WTSOpenServerExW(WCHAR *server_name)
+{
+    FIXME("(%s) stub\n", debugstr_w(server_name));
+    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+    return NULL;
+}
+
+/************************************************************
+ *                WTSOpenServerExA (WTSAPI32.@)
+ */
+HANDLE WINAPI WTSOpenServerExA(char *server_name)
+{
+    FIXME("(%s) stub\n", debugstr_a(server_name));
+    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+    return NULL;
+}
+
 /************************************************************
  *                WTSOpenServerA (WTSAPI32.@)
  */
@@ -254,8 +336,17 @@ BOOL WINAPI WTSQuerySessionInformationW(
  */
 BOOL WINAPI WTSQueryUserToken(ULONG session_id, PHANDLE token)
 {
-    FIXME("%u %p\n", session_id, token);
-    return FALSE;
+    FIXME("%u %p semi-stub!\n", session_id, token);
+
+    if (!token)
+    {
+        SetLastError(ERROR_INVALID_PARAMETER);
+        return FALSE;
+    }
+
+    return DuplicateHandle(GetCurrentProcess(), GetCurrentProcessToken(),
+                           GetCurrentProcess(), token,
+                           0, FALSE, DUPLICATE_SAME_ACCESS);
 }
 
 /************************************************************
