@@ -943,7 +943,7 @@ int get_cpu_from_name( const char *name )
 {
     unsigned int i;
 
-    for (i = 0; i < sizeof(cpu_names)/sizeof(cpu_names[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(cpu_names); i++)
         if (!strcmp( cpu_names[i].name, name )) return cpu_names[i].cpu;
     return -1;
 }
@@ -1093,7 +1093,7 @@ const char *func_declaration( const char *func )
         return "";
     case PLATFORM_WINDOWS:
         free( buffer );
-        buffer = strmake( ".def %s%s; .scl 2; .type 32; .endef", target_cpu == CPU_x86 ? "_" : "", func );
+        buffer = strmake( ".def %s; .scl 2; .type 32; .endef", asm_name(func) );
         break;
     default:
         free( buffer );
