@@ -208,6 +208,7 @@ struct options
     int wine_builtin;
     int unwind_tables;
     int strip;
+    int pic;
     const char* wine_objdir;
     const char* output_name;
     const char* image_base;
@@ -382,6 +383,7 @@ static int try_link( const strarray *prefix, const strarray *link_tool, const ch
 
 static strarray *get_link_args( struct options *opts, const char *output_name )
 {
+    int use_wine_crt = opts->wine_builtin && opts->shared;
     const strarray *link_tool = get_translator( opts );
     strarray *flags = strarray_alloc();
     unsigned int i;
@@ -428,7 +430,7 @@ static strarray *get_link_args( struct options *opts, const char *output_name )
 
     case PLATFORM_WINDOWS:
     case PLATFORM_CYGWIN:
-        if (opts->shared)
+        if (opts->shared || opts->win16_app)
         {
             strarray_add( flags, "-shared" );
             strarray_add( flags, "-Wl,--kill-at" );
@@ -436,8 +438,8 @@ static strarray *get_link_args( struct options *opts, const char *output_name )
         else strarray_add( flags, opts->gui_app ? "-mwindows" : "-mconsole" );
 
         if (opts->unicode_app) strarray_add( flags, "-municode" );
-        if (opts->nodefaultlibs) strarray_add( flags, "-nodefaultlibs" );
-        if (opts->nostartfiles) strarray_add( flags, "-nostartfiles" );
+        if (opts->nodefaultlibs || use_wine_crt) strarray_add( flags, "-nodefaultlibs" );
+        if (opts->nostartfiles || use_wine_crt) strarray_add( flags, "-nostartfiles" );
 
         if (opts->subsystem)
         {
@@ -449,11 +451,16 @@ static strarray *get_link_args( struct options *opts, const char *output_name )
             }
         }
 
+        strarray_add( flags, "-Wl,--nxcompat" );
+
         if (opts->image_base) strarray_add( flags, strmake("-Wl,--image-base,%s", opts->image_base ));
 
         if (opts->large_address_aware && opts->target_cpu == CPU_x86)
             strarray_add( flags, "-Wl,--large-address-aware" );
 
+        /* make sure we don't need a libgcc_s dll on Windows */
+        strarray_add( flags, "-static-libgcc" );
+
         return flags;
 
     default:
@@ -473,6 +480,8 @@ static strarray *get_link_args( struct options *opts, const char *output_name )
 
     strarray_add( flags, "-shared" );
     strarray_add( flags, "-Wl,-Bsymbolic" );
+    if (!opts->noshortwchar && opts->target_cpu == CPU_ARM)
+        strarray_add( flags, "-Wl,--no-wchar-size-warning" );
 
     /* Try all options first - this is likely to succeed on modern compilers */
     if (!try_link( opts->prefix, link_tool, "-fPIC -shared -Wl,-Bsymbolic "
@@ -684,7 +693,10 @@ static void compile(struct options* opts, const char* lang)
             strarray_add(comp_args, "-DWINE_UNICODE_NATIVE");
 	}
         strarray_add(comp_args, "-D_REENTRANT");
-        strarray_add(comp_args, "-fPIC");
+        if (opts->pic)
+            strarray_add(comp_args, "-fPIC");
+        else
+            strarray_add(comp_args, "-fno-PIC");
     }
 
     if (opts->target_cpu == CPU_x86_64 || opts->target_cpu == CPU_ARM64)
@@ -836,6 +848,7 @@ static strarray *get_winebuild_args(struct options *opts)
         strarray_add( spec_args, "--target" );
         strarray_add( spec_args, opts->target );
     }
+    if (!opts->use_msvcrt) strarray_add( spec_args, "-munix" );
     if (opts->unwind_tables) strarray_add( spec_args, "-fasynchronous-unwind-tables" );
     else strarray_add( spec_args, "-fno-asynchronous-unwind-tables" );
     return spec_args;
@@ -885,6 +898,7 @@ static void build(struct options* opts)
     const char *spec_o_name;
     const char *output_name, *spec_file, *lang;
     int generate_app_loader = 1;
+    const char* crt_lib = NULL;
     int fake_module = 0;
     int is_pe = (opts->target_platform == PLATFORM_WINDOWS || opts->target_platform == PLATFORM_CYGWIN);
     unsigned int j;
@@ -970,8 +984,15 @@ static void build(struct options* opts)
 		    strarray_add(files, strmake("-o%s", file));
 		    break;
 		case file_arh:
-		    strarray_add(files, strmake("-a%s", file));
-		    break;
+                    if (opts->use_msvcrt)
+                    {
+                        const char *p = strrchr(file, '/');
+                        if (p) p++;
+                        else p = file;
+                        if (!strncmp(p, "libmsvcr", 8) || !strncmp(p, "libucrt", 7)) crt_lib = file;
+                    }
+                    strarray_add(files, strmake("-a%s", file));
+                    break;
 		case file_so:
 		    strarray_add(files, strmake("-s%s", file));
 		    break;
@@ -990,9 +1011,6 @@ static void build(struct options* opts)
 	    lang = file;
     }
 
-    if (opts->win16_app && is_pe)
-        error( "Building 16-bit code is not supported for Windows\n" );
-
     /* add the default libraries, if needed */
 
     if (!opts->wine_objdir && !opts->nodefaultlibs)
@@ -1009,8 +1027,12 @@ static void build(struct options* opts)
 
     if (!opts->nodefaultlibs)
     {
-        if (opts->use_msvcrt) add_library(opts, lib_dirs, files, "msvcrt");
         add_library(opts, lib_dirs, files, "winecrt0");
+        if (opts->use_msvcrt)
+        {
+            if (!crt_lib) add_library(opts, lib_dirs, files, "msvcrt");
+            else strarray_add(files, strmake("-a%s", crt_lib));
+        }
         if (opts->win16_app) add_library(opts, lib_dirs, files, "kernel");
         add_library(opts, lib_dirs, files, "kernel32");
         add_library(opts, lib_dirs, files, "ntdll");
@@ -1026,7 +1048,7 @@ static void build(struct options* opts)
     if (opts->force_pointer_size)
         strarray_add(spec_args, strmake("-m%u", 8 * opts->force_pointer_size ));
     strarray_add(spec_args, "-D_REENTRANT");
-    strarray_add(spec_args, "-fPIC");
+    if (opts->pic && !is_pe) strarray_add(spec_args, "-fPIC");
     strarray_add(spec_args, opts->shared ? "--dll" : "--exe");
     if (fake_module)
     {
@@ -1151,6 +1173,8 @@ static void build(struct options* opts)
 	strarray_add(link_args, "-lc");
     }
 
+    if ((opts->nodefaultlibs || opts->shared) && is_pe) strarray_add( link_args, "-lgcc" );
+
     spawn(opts->prefix, link_args, 0);
     strarray_free (link_args);
 
@@ -1345,6 +1369,7 @@ int main(int argc, char **argv)
     opts.linker_args = strarray_alloc();
     opts.compiler_args = strarray_alloc();
     opts.winebuild_args = strarray_alloc();
+    opts.pic = 1;
 
     /* determine the processor type */
     if (strendswith(argv[0], "winecpp")) opts.processor = proc_cpp;
@@ -1466,6 +1491,10 @@ int main(int argc, char **argv)
                         opts.unwind_tables = 1;
 		    else if (!strcmp("-fno-asynchronous-unwind-tables", argv[i]))
                         opts.unwind_tables = 0;
+                    else if (!strcmp("-fPIC", argv[i]) || !strcmp("-fpic", argv[i]))
+                        opts.pic = 1;
+                    else if (!strcmp("-fno-PIC", argv[i]) || !strcmp("-fno-pic", argv[i]))
+                        opts.pic = 0;
 		    break;
 		case 'l':
 		    strarray_add(opts.files, strmake("-l%s", option_arg));
